@@ -4,7 +4,7 @@
  *
  * This file is part of the GNOME Devtools Libraries.
  *
- * Copyright (C) 2002 Gustavo Girldez <gustavo.giraldez@gmx.net>
+ * Copyright (C) 2002 Gustavo Giráldez <gustavo.giraldez@gmx.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -111,6 +111,10 @@ static void     gdl_dock_master_layout_changed (GdlDockMaster     *master);
 
 static void gdl_dock_master_set_switcher_style (GdlDockMaster *master,
                                                 GdlSwitcherStyle switcher_style);
+static void gdl_dock_master_set_tab_pos        (GdlDockMaster     *master,
+                                                GtkPositionType    pos);
+static void gdl_dock_master_set_tab_reorderable (GdlDockMaster    *master,
+                                                gboolean           reorderable);
 
 /* ----- Private data types and variables ----- */
 
@@ -118,7 +122,9 @@ enum {
     PROP_0,
     PROP_DEFAULT_TITLE,
     PROP_LOCKED,
-    PROP_SWITCHER_STYLE
+    PROP_SWITCHER_STYLE,
+    PROP_TAB_POS,
+    PROP_TAB_REORDERABLE
 };
 
 enum {
@@ -152,6 +158,8 @@ struct _GdlDockMasterPrivate {
     GHashTable     *unlocked_items;
 
     GdlSwitcherStyle switcher_style;
+    GtkPositionType tab_pos;
+    gboolean        tab_reorderable;
 
     /* Window for preview rect */
     GtkWidget* area_window;
@@ -161,6 +169,9 @@ struct _GdlDockMasterPrivate {
     (g_hash_table_size ((master)->priv->unlocked_items) == 0 ? 1 :     \
      (g_hash_table_size ((master)->priv->locked_items) == 0 ? 0 : -1))
 
+#define GBOOLEAN_TO_POINTER(i) (GINT_TO_POINTER ((i) ? 2 : 1))
+#define GPOINTER_TO_BOOLEAN(i) ((gboolean) ((GPOINTER_TO_INT(i) == 2) ? TRUE : FALSE))
+
 static guint master_signals [LAST_SIGNAL] = { 0 };
 
 
@@ -206,6 +217,21 @@ gdl_dock_master_class_init (GdlDockMasterClass *klass)
                            GDL_SWITCHER_STYLE_BOTH,
                            G_PARAM_READWRITE));
 
+    g_object_class_install_property (
+        object_class, PROP_TAB_POS,
+        g_param_spec_enum ("tab-pos", _("Tab Position"),
+                           _("Which side of the notebook holds the tabs"),
+                           GTK_TYPE_POSITION_TYPE,
+                           GTK_POS_BOTTOM,
+                           G_PARAM_READWRITE));
+
+    g_object_class_install_property (
+        object_class, PROP_TAB_REORDERABLE,
+        g_param_spec_boolean ("tab-reorderable", _("Tab reorderable"),
+                              _("Whether the tab is reorderable by user action"),
+                              FALSE,
+                              G_PARAM_READWRITE));
+
     /**
      * GdlDockMaster::layout-changed:
      *
@@ -241,6 +267,8 @@ gdl_dock_master_init (GdlDockMaster *master)
 
     master->priv->number = 1;
     master->priv->switcher_style = GDL_SWITCHER_STYLE_BOTH;
+    master->priv->tab_pos = GTK_POS_BOTTOM;
+    master->priv->tab_reorderable = FALSE;
     master->priv->locked_items = g_hash_table_new (g_direct_hash, g_direct_equal);
     master->priv->unlocked_items = g_hash_table_new (g_direct_hash, g_direct_equal);
 }
@@ -426,6 +454,12 @@ gdl_dock_master_set_property  (GObject      *object,
         case PROP_SWITCHER_STYLE:
             gdl_dock_master_set_switcher_style (master, g_value_get_enum (value));
             break;
+        case PROP_TAB_POS:
+            gdl_dock_master_set_tab_pos (master, g_value_get_enum (value));
+            break;
+        case PROP_TAB_REORDERABLE:
+            gdl_dock_master_set_tab_reorderable (master, g_value_get_boolean (value));
+            break;
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
             break;
@@ -450,6 +484,12 @@ gdl_dock_master_get_property  (GObject      *object,
         case PROP_SWITCHER_STYLE:
             g_value_set_enum (value, master->priv->switcher_style);
             break;
+        case PROP_TAB_POS:
+            g_value_set_enum (value, master->priv->tab_pos);
+            break;
+        case PROP_TAB_REORDERABLE:
+            g_value_set_enum (value, master->priv->tab_reorderable);
+            break;
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
             break;
@@ -883,12 +923,18 @@ gdl_dock_master_add (GdlDockMaster *master,
             item_notify_cb (object, NULL, master);
         }
 
-        /* If the item is notebook, set the switcher style */
+        /* If the item is notebook, set the switcher style and notebook
+         * settings. */
         if (GDL_IS_DOCK_NOTEBOOK (object) &&
             GDL_IS_SWITCHER (gdl_dock_item_get_child (GDL_DOCK_ITEM (object))))
         {
-            g_object_set (gdl_dock_item_get_child (GDL_DOCK_ITEM (object)), "switcher-style",
+            GtkWidget *child = gdl_dock_item_get_child (GDL_DOCK_ITEM (object));
+            g_object_set (child, "switcher-style",
                           master->priv->switcher_style, NULL);
+            g_object_set (child, "tab-pos",
+                          master->priv->tab_pos, NULL);
+            g_object_set (child, "tab-reorderable",
+                          master->priv->tab_reorderable, NULL);
         }
 
         /* post a layout_changed emission if the item is not automatic
@@ -1131,3 +1177,90 @@ gdl_dock_master_set_switcher_style (GdlDockMaster *master,
                              GINT_TO_POINTER (switcher_style));
 }
 
+static void
+set_tab_pos_foreach (GtkWidget *obj, gpointer user_data)
+{
+    GtkPositionType tab_pos = GPOINTER_TO_INT (user_data);
+
+    if (!GDL_IS_DOCK_ITEM (obj))
+        return;
+
+    if (GDL_IS_DOCK_NOTEBOOK (obj)) {
+
+        GtkWidget *child = gdl_dock_item_get_child (GDL_DOCK_ITEM (obj));
+        if (GDL_IS_SWITCHER (child)) {
+
+            g_object_set (child, "tab-pos", tab_pos, NULL);
+        }
+    } else if (gdl_dock_object_is_compound (GDL_DOCK_OBJECT (obj))) {
+
+        gtk_container_foreach (GTK_CONTAINER (obj),
+                               set_tab_pos_foreach,
+                               user_data);
+    }
+}
+
+static void
+gdl_dock_master_set_tab_pos (GdlDockMaster *master,
+                             GtkPositionType tab_pos)
+{
+    GList *l;
+    g_return_if_fail (GDL_IS_DOCK_MASTER (master));
+
+    master->priv->tab_pos = tab_pos;
+    for (l = master->priv->toplevel_docks; l; l = l->next) {
+        GdlDock *dock = GDL_DOCK (l->data);
+        GtkWidget *root = GTK_WIDGET (gdl_dock_get_root (dock));
+        if (root != NULL)
+            set_tab_pos_foreach (root,
+                                 GINT_TO_POINTER (tab_pos));
+    }
+
+    /* just to be sure hidden items are set too */
+    gdl_dock_master_foreach (master, (GFunc) set_tab_pos_foreach,
+                             GINT_TO_POINTER (tab_pos));
+}
+
+static void
+set_tab_reorderable_foreach (GtkWidget *obj, gpointer user_data)
+{
+    gboolean tab_reorderable = GPOINTER_TO_BOOLEAN (user_data);
+
+    if (!GDL_IS_DOCK_ITEM (obj))
+        return;
+
+    if (GDL_IS_DOCK_NOTEBOOK (obj)) {
+
+        GtkWidget *child = gdl_dock_item_get_child (GDL_DOCK_ITEM (obj));
+        if (GDL_IS_SWITCHER (child)) {
+
+            g_object_set (child, "tab-reorderable", tab_reorderable, NULL);
+        }
+    } else if (gdl_dock_object_is_compound (GDL_DOCK_OBJECT (obj))) {
+
+        gtk_container_foreach (GTK_CONTAINER (obj),
+                               set_tab_reorderable_foreach,
+                               user_data);
+    }
+}
+
+static void
+gdl_dock_master_set_tab_reorderable (GdlDockMaster *master,
+                                     gboolean tab_reorderable)
+{
+    GList *l;
+    g_return_if_fail (GDL_IS_DOCK_MASTER (master));
+
+    master->priv->tab_reorderable = tab_reorderable;
+    for (l = master->priv->toplevel_docks; l; l = l->next) {
+        GdlDock *dock = GDL_DOCK (l->data);
+        GtkWidget *root = GTK_WIDGET (gdl_dock_get_root (dock));
+        if (root != NULL)
+            set_tab_reorderable_foreach (root,
+                                         GBOOLEAN_TO_POINTER (tab_reorderable));
+    }
+
+    /* just to be sure hidden items are set too */
+    gdl_dock_master_foreach (master, (GFunc) set_tab_reorderable_foreach,
+                             GBOOLEAN_TO_POINTER (tab_reorderable));
+}
